// ----------------------------------------------------------------------------
//
// Snoop Component Suite version 3.0
//
// http://www.netlab.co.kr
//
// Copyright (c) NetworkLab Corporation All rights reserved
//
// ----------------------------------------------------------------------------

unit SnoopTCPOption;

interface

uses
  Windows, Classes, SysUtils, Snoop;

const
  SNOOP_TCP_OPTION_MEANING_SIZE = 256;

type
  PSnoopTCPOption = ^TSnoopTCPOption;
  TSnoopTCPOption = packed record
    Kind: UINT8;
    Length: UINT8;
    Value: Variant;
    Meaning: array[0 .. SNOOP_TCP_OPTION_MEANING_SIZE - 1] of Char;
  end;

//
// Return One TCPOption Bufer Size
//
function snoopGetTCPOption(
  TCPOption: PChar;
  TCPOptionLen: Integer;
  SnoopTCPOption: PSnoopTCPOption): Integer;

//
// Return Total TCPOption Count
//
function snoopGetTCPOptionList(
  TCPOption: PChar;
  TCPOptionLen: Integer;
  SnoopTCPOptionList: PSnoopTCPOption;
  MaxSnoopTCPOptionCount: Integer): Integer;

implementation

// ----------------------------------------------------------------------------
{ from : http://www.iana.org/assignments/tcp-parameters

TCP OPTION NUMBERS

(last updated 2007-02-15)

The Transmission Control Protocol (TCP) has provision for optional
header fields identified by an option kind field.  Options 0 and 1 are
exactly one octet which is their kind field.  All other options have
their one octet kind field, followed by a one octet length field,
followed by length-2 octets of option data.

Kind   Length   Meaning                           Reference
----   ------   -------------------------------   ---------
  0        -    End of Option List                 [RFC793]
  1        -    No-Operation                       [RFC793]
  2        4    Maximum Segment Size               [RFC793]
  3        3    WSOPT - Window Scale              [RFC1323]
  4        2    SACK Permitted                    [RFC2018]
  5        N    SACK                              [RFC2018]
  6        6    Echo (obsoleted by option 8)      [RFC1072]
  7        6    Echo Reply (obsoleted by option 8)[RFC1072]
  8       10    TSOPT - Time Stamp Option         [RFC1323]
  9        2    Partial Order Connection Permitted[RFC1693]
 10        3    Partial Order Service Profile     [RFC1693]
 11             CC                                [RFC1644]
 12             CC.NEW                            [RFC1644]
 13             CC.ECHO                           [RFC1644]
 14         3   TCP Alternate Checksum Request    [RFC1146]
 15         N   TCP Alternate Checksum Data       [RFC1146]
 16             Skeeter                           [Knowles]
 17             Bubba                             [Knowles]
 18         3   Trailer Checksum Option    [Subbu & Monroe]
 19        18   MD5 Signature Option              [RFC2385]
 20             SCPS Capabilities                   [Scott]
 21             Selective Negative Acknowledgements [Scott]
 22             Record Boundaries                   [Scott]
 23             Corruption experienced              [Scott]
 24             SNAP         [Sukonnik]
 25             Unassigned (released 12/18/00)
 26             TCP Compression Filter           [Bellovin]
 27          8  Quick-Start Response              [RFC4782]
 28-252         Unassigned
 253         N  RFC3692-style Experiment 1 (*)    [RFC4727]
 254         N  RFC3692-style Experiment 2 (*)    [RFC4727]

(*) It is only appropriate to use these values in explicitly-
    configured experiments; they MUST NOT be shipped as defaults in
    implementations.  See RFC 3692 for details.


TCP ALTERNATE CHECKSUM NUMBERS

Number  Description      Reference
------- ------------------------------- ----------
   0    TCP Checksum                    [RFC-1146]
   1    8-bit Fletchers's algorithm     [RFC-1146]
   2    16-bit Fletchers's algorithm    [RFC-1146]
   3    Redundant Checksum Avoidance    [Kay]

}
// ----------------------------------------------------------------------------

//
// Return One TCPOption Bufer Size
//
function snoopGetTCPOption(
  TCPOption: PChar;
  TCPOptionLen: Integer;
  SnoopTCPOption: PSnoopTCPOption): Integer;
label
  _error;
var
  p: PUINT8;

  Kind: UINT8;
  Length: UINT8;
  Meaning: String;
begin
  p := PUINT8(TCPOption);

  //
  // Set Kind
  //
  if TCPOptionLen < 1 then goto _error;
  Kind := p^; inc(p); dec(TCPOptionLen);
  SnoopTCPOption.Kind := Kind;

  //
  // Set Length and Value
  //
  if (Kind <> 0) and (Kind <> 1) then
  begin
    if TCPOptionLen < 1 then goto _error;
    Length := p^; inc(p, 1); dec(TCPOptionLen);
    SnoopTCPOption^.Length := Length;

    if TCPOptionLen < Length - 2 then goto _error;
    StrMove(PChar(@SnoopTCPOption^.Value), PChar(p), Length - 2);
    inc(p, Length - 2);
    // dec(TCPOptionLen, Length - 2); // remove warning
  end;

  //
  // Set Meaning
  //
  case Ord(Kind) of
      0: Meaning := 'EOL';
      1: Meaning := 'NOP';
      2: Meaning := 'MSS';
      3: Meaning := 'Window Scale';
      4: Meaning := 'SACK Permitted';
      5: Meaning := 'SACK';
      6: Meaning := 'Echo (obsoleted by option 8)';
      7: Meaning := 'Echo Reply (obsoleted by option 8';
      8: Meaning := 'TSOPT - Time Stamp Option';
      9: Meaning := 'Partial Order Connection Permitte';
     10: Meaning := 'Partial Order Service Profile';
     11: Meaning := 'CC';
     12: Meaning := 'CC.NEW';
     13: Meaning := 'CC.ECHO';
     14: Meaning := 'TCP Alternate Checksum Request';
     15: Meaning := 'TCP Alternate Checksum Data';
     16: Meaning := 'Skeeter';
     17: Meaning := 'Bubba';
     18: Meaning := 'Trailer Checksum Option    [Subbu';
     19: Meaning := 'MD5 Signature Option';
     20: Meaning := 'SCPS Capabilities';
     21: Meaning := 'Selective Negative Acknowledgements';
     22: Meaning := 'Record Boundaries';
     23: Meaning := 'Corruption experienced';
     24: Meaning := 'SNAP';
     25: Meaning := 'Unassigned (released 12/18/00)';
     26: Meaning := 'TCP Compression Filter';
     27: Meaning := 'Quick-Start Response';
    253: Meaning := 'RFC3692-style Experiment 1';
    254: Meaning := 'RFC3692-style Experiment 2';
    else Meaning := 'UNKNOWN';
  end;
  if Meaning = 'UNKNOWN' then
  begin
    Result := 0;
    exit;
  end;

  Meaning:= Copy(Meaning, 1, SNOOP_TCP_OPTION_MEANING_SIZE);
  FillMemory(@SnoopTCPOption.Meaning[0], SNOOP_TCP_OPTION_MEANING_SIZE, 0);
  StrMove(@SnoopTCPOption.Meaning[0], PChar(Meaning), System.Length(Meaning));

  Result := p - TCPOption;
  exit;
_error:
  Result := 0;
end;

//
// Return Total TCPOption Count
//
function snoopGetTCPOptionList(
  TCPOption: PChar;
  TCPOptionLen: Integer;
  SnoopTCPOptionList: PSnoopTCPOption;
  MaxSnoopTCPOptionCount: Integer): Integer;
var
  nRes, Count: Integer;
  SnoopTCPOption: PSnoopTCPOption;
begin
  Count := 0;
  SnoopTCPOption := SnoopTCPOptionList;
  while true do
  begin
    if Count > MaxSnoopTCPOptionCount then break;

    nRes := snoopGetTCPOption(TCPOption, TCPOptionLen, SnoopTCPOption);
    if nRes = 0 then break;
    
    inc(TCPOption, nRes);
    dec(TCPOptionLen, nRes);
    inc(SnoopTCPOption);
    inc(Count);
  end;
  Result := Count;
end;

end.
